#ifndef PORYGONLANG_TOKEN_HPP
#define PORYGONLANG_TOKEN_HPP

#include <utility>
#include <string>
#include "TokenKind.hpp"
using namespace std;

class IToken{
public:
    virtual TokenKind GetKind() = 0;
};

class SimpleToken : public IToken{
public:
    TokenKind Kind;

    explicit SimpleToken(TokenKind type){
        Kind = type;
    }

    TokenKind GetKind() override{
        return Kind;
    }
};

class IntegerToken : public IToken{
public:
    long Value;

    explicit IntegerToken(long value){
        Value = value;
    }

    TokenKind GetKind() override{
        return TokenKind::Integer;
    }
};

class FloatToken : public IToken{
public:
    double Value;

    explicit FloatToken(double value){
        Value = value;
    }

    TokenKind GetKind() override{
        return TokenKind::Float;
    }
};

class IdentifierToken : public IToken{
public:
    string Value;

    explicit IdentifierToken(string value){
        Value = std::move(value);
    }

    TokenKind GetKind() override{
        return TokenKind::Identifier;
    }
};

#endif //PORYGONLANG_TOKEN_HPP
